#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"


#define SPLITTER        ","

char* resultString;

int srm_abortfiles_func( char* endpoint, struct srm_abort_files_input *input ) {

    int status;
    int len;
    int i;

	struct srm_context context;
    struct srmv2_filestatus *output;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_abort_files() returns the number of files or -1 for error
     */
    status = srm_abort_files( &context, input, &output );
    if ( status < 0 ) {
        /*
         * Something went wrong while while executing SRM web service
         */
        if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
        }

        return status;
    }
    else {
        /*
         * Allocate memory for result string
         */
        resultString = (char *) calloc( 50000, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }

        /*
         * Fill result string
         */
        for ( i = 0; i < status; i++ ) {
            /*
             * Print all information about srmv2_filestatus
             */
            printSrmFileStatus( i, &resultString, output[ i ] );
        }
        return status;
    }
}

/**
 * \fn                  msiSrmAbortFiles
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint -  SRM manager URL
 * \param[in STR_MS_T]  surls - list of hosts URLs
 * \param[in STR_MS_T]  token - token generated by prior operation
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmAbortFiles( msParam_t *endpoint, msParam_t *surls, msParam_t *token, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;
    int nr_surls;

    char** surlsArray       = NULL;

    struct srm_abort_files_input input;


    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmAbortFiles: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL
         || surls == NULL
         || token == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || surls->type == NULL
         || token->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0
         || strcmp( token->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Split SURLs string up into separated items (each item contains one SURL)
     */
    nr_surls = splitString( surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s %s", INVALID_SURL_SPEC, CHK_CMD );
		return rei->status;
    }

    /*
     * Set up srm_abort_files_input data structure
     */
	input.nbfiles       = nr_surls;         // Number of SURLs in the array
	input.surls         = surlsArray;       // Array of SURLs
    input.reqtoken      = (char * ) token->inOutStruct;

    status = srm_abortfiles_func( (char* ) endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortFiles: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
