#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"

/**
 * \fn                  msiSrmAbortRequest
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  token - token generated by prior operation
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmAbortRequest( msParam_t *endpoint, msParam_t *token, ruleExecInfo_t *rei ) {

	int status;

    struct srm_context context;


    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmAbortRequest: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL
         || token == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortRequest: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || token->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortRequest: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( token->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortRequest: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Initialize context
     */
    initialize_srm_context( &context, (char* ) endpoint->inOutStruct );

    /*
     * Execute SRM command
     */
    status = srm_abort_request( &context, (char* ) token->inOutStruct );
    if ( status == -1 ) {
        rei->status = EXEC_CMD_ERROR;
        if ( context.errbuf != NULL ) {
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortRequest returned with error code\nError message: %s", context.errbuf );
        }
        else {
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmAbortRequest returned with error code" );
        }
    }
    else {
        rei->status = 1;
    }
    return rei->status;
}
