#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define SPLITTER            ","
#define CORRECT_SURL        "\nPlease check if you specified a correct SURL.\n"
#define NULL_PTR            "\n\nERROR: Some information not available"
#define SURL_HEADER         "SURL:              "
#define OWNER_ID_HEADER     "Owner ID:          "
#define USER_ID_HEADER      "User ID:  "
#define GROUP_ID_HEADER     "Group ID: "
#define PERMISSIONS_HEADER  "   Permissions: "
#define OWNER_PERM_HEADER   "Owner permissions: "
#define USER_PERM_HEADER    "User permissions:  "
#define GROUP_PERM_HEADER   "Group permissions: "
#define OTHER_PERM_HEADER   "Other permissions: "
#define NULL_PTR_EXCEPTION  -10
#define DIGITS              6


char* resultString;

int srm_getpermission_func( char *endpoint, struct srm_getpermission_input *input ) {

    int user_permissions_count;
    int group_permissions_count;
    int status;
    int len;
    int i;
    int j;

    char loopNr[DIGITS];

	struct srm_context context;
	struct srm_getpermission_output output_gp;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_getpermission() returns the number of file permissions for success or -1 for error
     */
    status = srm_getpermission( &context, input, &output_gp );

    if ( status < 0 ) {
        /*
         * srm_getpermission() does not finished successfully, try multiple ways to get a meaningful error message
         */
        if ( output_gp.retstatus != NULL ) {
            /*
             * If a connection could be stablished and a return status was set, use this information for error message
             */
            len = strlen(ERROR_INTRO) + strlen(output_gp.retstatus->explanation) + strlen(".\n") + strlen(CORRECT_SURL);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s.\n%s", ERROR_INTRO, output_gp.retstatus->explanation, CORRECT_SURL );
        }
        else if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            /*
             * If there is no return status, something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
        }
    }
    else {
        if ( output_gp.permissions == NULL ) {
            return NULL_PTR_EXCEPTION;
        }

        /*
         * Allocate memory for result string length
         */
        resultString = calloc( 5000, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }

        strncat( resultString, "\n", strlen("\n") );

        /*
         * If the user specified multiple SURLs, there will be also multiple items in the response
         * and we have to process each.
         */
        for ( i = 0; i < status; i++ ) {
            if ( output_gp.permissions[ i ].surl != NULL
                 && output_gp.permissions[ i ].owner != NULL ) {

                user_permissions_count = output_gp.permissions[ i ].user_permissions_count;
                group_permissions_count = output_gp.permissions[ i ].group_permissions_count;                

                /*
                 * Create result string (SURL, ID of owner, owner, group, user, and other permissions)
                 */
                // SURL + owner ID
                strncat( resultString, SURL_HEADER, strlen(SURL_HEADER) );
                strncat( resultString, output_gp.permissions[ i ].surl, strlen(output_gp.permissions[ i ].surl) );
                strncat( resultString, "\n", strlen("\n") );
                strncat( resultString, OWNER_ID_HEADER, strlen(OWNER_ID_HEADER) );
                strncat( resultString, output_gp.permissions[ i ].owner, strlen(output_gp.permissions[ i ].owner) );
                strncat( resultString, "\n", strlen("\n") );

                // Owner permission
                strncat( resultString, OWNER_PERM_HEADER, strlen(OWNER_PERM_HEADER) );
                switch (output_gp.permissions[ i ].owner_permission) {
                    case SRM_PERMISSION_NONE:   strncat( resultString, "-", strlen("-") );
                                                break;
                    case SRM_PERMISSION_X:      strncat( resultString, "X", strlen("X") );
                                                break;
                    case SRM_PERMISSION_W:      strncat( resultString, "W", strlen("W") );
                                                break;
                    case SRM_PERMISSION_WX:     strncat( resultString, "WX", strlen("WX") );
                                                break;
                    case SRM_PERMISSION_R:      strncat( resultString, "R", strlen("R") );
                                                break;
                    case SRM_PERMISSION_RX:     strncat( resultString, "RX", strlen("RX") );
                                                break;
                    case SRM_PERMISSION_RW:     strncat( resultString, "RW", strlen("RW") );
                                                break;
                    case SRM_PERMISSION_RWX:    strncat( resultString, "RWX", strlen("RWX") );
                                                break;
                    default:                    strncat( resultString, "err", strlen("err") );
                }
                strncat( resultString, "\n", strlen("\n") );

                // User permission
                strncat( resultString, USER_PERM_HEADER, strlen(USER_PERM_HEADER) );
                strncat( resultString, "\n", strlen("\n") );
                for( j = 0; j < user_permissions_count; j++ ) {
                    strncat( resultString, "   ", strlen("   ") );
                    strncat( resultString, USER_ID_HEADER, strlen(USER_ID_HEADER) );
                    strncat( resultString, output_gp.permissions[ i ].user_permissions[ j ].name_id, strlen(output_gp.permissions[ i ].user_permissions[ j ].name_id) );
                    strncat( resultString, PERMISSIONS_HEADER, strlen(PERMISSIONS_HEADER) );
                    switch (output_gp.permissions[ i ].user_permissions[ j ].mode) {
                        case SRM_PERMISSION_NONE:   strncat( resultString, "-", strlen("-") );
                                                    break;
                        case SRM_PERMISSION_X:      strncat( resultString, "X", strlen("X") );
                                                    break;
                        case SRM_PERMISSION_W:      strncat( resultString, "W", strlen("W") );
                                                    break;
                        case SRM_PERMISSION_WX:     strncat( resultString, "WX", strlen("WX") );
                                                    break;
                        case SRM_PERMISSION_R:      strncat( resultString, "R", strlen("R") );
                                                    break;
                        case SRM_PERMISSION_RX:     strncat( resultString, "RX", strlen("RX") );
                                                    break;
                        case SRM_PERMISSION_RW:     strncat( resultString, "RW", strlen("RW") );
                                                    break;
                        case SRM_PERMISSION_RWX:    strncat( resultString, "RWX", strlen("RWX") );
                                                    break;
                        default:                    strncat( resultString, "err", strlen("err") );
                    }
                    strncat( resultString, "\n", strlen("\n") );
                }

                // Group permission
                strncat( resultString, GROUP_PERM_HEADER, strlen(GROUP_PERM_HEADER) );
                strncat( resultString, "\n", strlen("\n") );
                for( j = 0; j < group_permissions_count; j++ ) {
                    strncat( resultString, "   ", strlen("   ") );
                    strncat( resultString, GROUP_ID_HEADER, strlen(GROUP_ID_HEADER) );
                    strncat( resultString, output_gp.permissions[ i ].group_permissions[ j ].name_id, strlen(output_gp.permissions[ i ].group_permissions[ j ].name_id) );
                    strncat( resultString, PERMISSIONS_HEADER, strlen(PERMISSIONS_HEADER) );
                    switch (output_gp.permissions[ i ].group_permissions[ j ].mode) {
                        case SRM_PERMISSION_NONE:   strncat( resultString, "-", strlen("-") );
                                                    break;
                        case SRM_PERMISSION_X:      strncat( resultString, "X", strlen("X") );
                                                    break;
                        case SRM_PERMISSION_W:      strncat( resultString, "W", strlen("W") );
                                                    break;
                        case SRM_PERMISSION_WX:     strncat( resultString, "WX", strlen("WX") );
                                                    break;
                        case SRM_PERMISSION_R:      strncat( resultString, "R", strlen("R") );
                                                    break;
                        case SRM_PERMISSION_RX:     strncat( resultString, "RX", strlen("RX") );
                                                    break;
                        case SRM_PERMISSION_RW:     strncat( resultString, "RW", strlen("RW") );
                                                    break;
                        case SRM_PERMISSION_RWX:    strncat( resultString, "RWX", strlen("RWX") );
                                                    break;
                        default:                    strncat( resultString, "err", strlen("err") );
                    }
                    strncat( resultString, "\n", strlen("\n") );
                }

                // Other permission
                strncat( resultString, OTHER_PERM_HEADER, strlen(OTHER_PERM_HEADER) );
                switch (output_gp.permissions[ i ].other_permission) {
                    case SRM_PERMISSION_NONE:   strncat( resultString, "-", strlen("-") );
                                                break;
                    case SRM_PERMISSION_X:      strncat( resultString, "X", strlen("X") );
                                                break;
                    case SRM_PERMISSION_W:      strncat( resultString, "W", strlen("W") );
                                                break;
                    case SRM_PERMISSION_WX:     strncat( resultString, "WX", strlen("WX") );
                                                break;
                    case SRM_PERMISSION_R:      strncat( resultString, "R", strlen("R") );
                                                break;
                    case SRM_PERMISSION_RX:     strncat( resultString, "RX", strlen("RX") );
                                                break;
                    case SRM_PERMISSION_RW:     strncat( resultString, "RW", strlen("RW") );
                                                break;
                    case SRM_PERMISSION_RWX:    strncat( resultString, "RWX", strlen("RWX") );
                                                break;
                    default:                    strncat( resultString, "err", strlen("err") );
                }
                strncat( resultString, "\n", strlen("\n") );
            }
            else {
                strncat( resultString, NULL_PTR, strlen(NULL_PTR) );
                strncat( resultString, " for SURL number ", strlen(" for SURL number ") );
                snprintf( loopNr, DIGITS, "%d\n", i+1 );
                strncat( resultString, loopNr, strlen(loopNr) );
            }
            strncat( resultString, "\n", strlen("\n") );
        }
    }
    return status;
}

/**
 * \fn                  msiSrmGetPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surls - list of storage URLs
 * \param[out STR_MS_T] result - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmGetPermission( msParam_t *endpoint, msParam_t *surls, msParam_t *result, ruleExecInfo_t *rei ) {

    int status;
    int nr_surls;

    char** surlsArray = NULL;

	struct srm_getpermission_input gp_input;
	
    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmGetPermission: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointers
     */
    if ( endpoint == NULL
         || surls == NULL
         || result == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || surls->type == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0 ) {

		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    /*
     * Get separated SURLs from string
     */
    nr_surls = splitString( (char* ) surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {

   		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s %s", INVALID_SURL_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_getpermission_input data structure
     */
	gp_input.nbfiles        = nr_surls;         // Number of SURLs in the array
	gp_input.surls          = surlsArray;       // Array of SURLs

    status = srm_getpermission_func( (char *) endpoint->inOutStruct, &gp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s", OUT_OF_MEM );
    }
    else if ( status == NULL_PTR_EXCEPTION ) {
        /* NO RESULTS RECEIVED */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: Response does not contain any permission information." );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( result, resultString );
    }
    return rei->status;
}
