#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define SPLITTER                ","
#define SRM_LONG64_DIGITS       20
#define INT_DIGITS              10
#define DEFAULT_ERROR_MSG       "srm_getspacemd() failed.\n"
#define TOKEN_HEADER            "\nSpace Tokens:        "
#define OWNER_HEADER            "\nOwner:               "
#define TOT_SIZE_HEADER         "\nTotal Size:          "
#define GUAR_SIZE_HEADER        "\nGuaranteed Size:     "
#define UNUSED_SIZE_HEADER      "\nUnused Size:         "
#define ASSIGNED_LT_HEADER      "\nAssigned Life Time:  "
#define LEFT_LT_HEADER          "\nLeft Life Time:      "
#define RET_POLICY_HEADER       "\nRetention Policy:    "
#define ACCESS_LAT_HEADER       "\nAccess Latency:      "
#define RET_POL_UNKNOWN         "unknown"
#define RET_POL_REPLICA         "Replica"
#define RET_POL_OUTPUT          "Output"
#define RET_POL_CUSTODIAL       "Custodial"
#define RET_POL_STR_MAX_SIZE    9
#define LAT_UNKNOWN             "unknown"
#define LAT_ONLINE              "Online"
#define LAT_NEARLINE            "Nearline"
#define LATENCY_STR_MAX_SIZE    8
#define TOKEN_ERROR             "No information available"


char* resultString;

int srm_getspacemd_func( char* endpoint, struct srm_getspacemd_input *input ) {

    int status;
    int len;
    int i;

    char* toString;
    char* tmp;

	struct srm_context context;
    struct srm_spacemd *output;

    toString = malloc( (SRM_LONG64_DIGITS + 1) * sizeof(char) );
    if ( toString == NULL ) {
        return OUT_OF_MEMORY_EXCEPTION;
    }
    
    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_getspacemd() returns spce meta data for requested space tokens or -1 for error
     */
    status = srm_getspacemd( &context, input, &output );
    if ( status < 0 ) {
        /*
         * Error case
         */
        if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(DEFAULT_ERROR_MSG) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n", ERROR_INTRO, DEFAULT_ERROR_MSG, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(DEFAULT_ERROR_MSG);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, DEFAULT_ERROR_MSG );
        }
        return status;
    }
    else {
        len = 0;
        for ( i = 0; i < status; i++ ) {
            /*
             * Compute result string length and allocate memory
             */
            if ( output[ i ].spacetoken != NULL ) {
                len = len + strlen(output[ i ].spacetoken);
            }
            else {
                len = strlen(TOKEN_ERROR);
            }
            if ( output[ i ].owner != NULL ) {
                len = len + strlen(output[ i ].owner);
            }
            else {
                len = len + strlen(TOKEN_ERROR);
            }
            len = len  + strlen(TOKEN_HEADER)
                       + strlen(OWNER_HEADER)
                       + strlen(TOT_SIZE_HEADER) + SRM_LONG64_DIGITS
                       + strlen(GUAR_SIZE_HEADER) + SRM_LONG64_DIGITS
                       + strlen(UNUSED_SIZE_HEADER) + SRM_LONG64_DIGITS
                       + strlen(ASSIGNED_LT_HEADER) + INT_DIGITS
                       + strlen(LEFT_LT_HEADER) + INT_DIGITS
                       + strlen(RET_POLICY_HEADER) + RET_POL_STR_MAX_SIZE
                       + strlen(ACCESS_LAT_HEADER) + LATENCY_STR_MAX_SIZE;
            tmp = (char *) realloc( resultString, (len + 1) * sizeof(char) );
            if ( tmp == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            else {
                resultString = tmp;
                tmp = NULL;
            }

            /*
             * Print returned meta data information
             */
            strncat( resultString, TOKEN_HEADER, strlen(TOKEN_HEADER) );
            if ( output[ i ].spacetoken != NULL ) {
                strncat( resultString, output[ i ].spacetoken, strlen(output[ i ].spacetoken) );
            }
            else {
                strncat( resultString, TOKEN_ERROR, strlen(TOKEN_ERROR) );
            }
            strncat( resultString, OWNER_HEADER, strlen(OWNER_HEADER) );
            if ( output[ i ].owner != NULL ) {
                strncat( resultString, output[ i ].owner, strlen(output[ i ].owner) );
            }
            else {
                strncat( resultString, TOKEN_ERROR, strlen(TOKEN_ERROR) );
            }

            strncat( resultString, TOT_SIZE_HEADER, strlen(TOT_SIZE_HEADER) );
            snprintf( toString, SRM_LONG64_DIGITS, "%lli", (long long int)output[ i ].totalsize );
            strncat( resultString, toString, strlen(toString) );

            strncat( resultString, GUAR_SIZE_HEADER, strlen(GUAR_SIZE_HEADER) );
            snprintf( toString, SRM_LONG64_DIGITS, "%lli", (long long int)output[ i ].guaranteedsize );
            strncat( resultString, toString, strlen(toString) );

            strncat( resultString, UNUSED_SIZE_HEADER, strlen(UNUSED_SIZE_HEADER) );
            snprintf( toString, SRM_LONG64_DIGITS, "%lli", (long long int)output[ i ].unusedsize );
            strncat( resultString, toString, strlen(toString) );

            strncat( resultString, ASSIGNED_LT_HEADER, strlen(ASSIGNED_LT_HEADER) );
            snprintf( toString, INT_DIGITS, "%d", output[ i ].lifetimeassigned );
            strncat( resultString, toString, strlen(toString) );

            strncat( resultString, LEFT_LT_HEADER, strlen(LEFT_LT_HEADER) );
            snprintf( toString, INT_DIGITS, "%d", output[ i ].lifetimeleft );
            strncat( resultString, toString, strlen(toString) );

            strncat( resultString, RET_POLICY_HEADER, strlen(RET_POLICY_HEADER) );
            if ( output[ i ].retentionpolicy == SRM_POLICY_UNKNOWN ) {
                strncat( resultString, RET_POL_UNKNOWN, strlen(RET_POL_UNKNOWN) );
            } else if ( output[ i ].retentionpolicy == SRM_POLICY_REPLICA ) {
                strncat( resultString, RET_POL_REPLICA, strlen(RET_POL_REPLICA) );
            } else if ( output[ i ].retentionpolicy == SRM_POLICY_OUTPUT ) {
                strncat( resultString, RET_POL_OUTPUT, strlen(RET_POL_OUTPUT) );
            } else if ( output[ i ].retentionpolicy == SRM_POLICY_CUSTODIAL ) {
                strncat( resultString, RET_POL_CUSTODIAL, strlen(RET_POL_CUSTODIAL) );
            } else {
                strncat( resultString, "---", strlen("---") );
            }

            strncat( resultString, ACCESS_LAT_HEADER, strlen(ACCESS_LAT_HEADER) );
            if ( output[ i ].accesslatency == SRM_LATENCY_UNKNOWN ) {
                strncat( resultString, LAT_UNKNOWN, strlen(LAT_UNKNOWN) );
            } else if ( output[ i ].accesslatency == SRM_LATENCY_ONLINE ) {
                strncat( resultString, LAT_ONLINE, strlen(LAT_ONLINE) );
            } else if ( output[ i ].accesslatency == SRM_LATENCY_NEARLINE ) {
                strncat( resultString, LAT_NEARLINE, strlen(LAT_NEARLINE) );
            } else {
                strncat( resultString, "---", strlen("---") );
            }
        }
        return status;
    }
}

/**
 * \fn                  msiSrmGetSpaceMD
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  spacetokens - list of space tokens
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmGetSpaceMD( msParam_t *endpoint, msParam_t *spacetokens, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;
    int nr_tokens;

    char** spacetokensArray = NULL;

    struct srm_getspacemd_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmGetSpaceMD: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL 
         || spacetokens == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || spacetokens->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: ", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( spacetokens->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Fill input data structure
     */
    nr_tokens = splitString( (char* ) spacetokens->inOutStruct, &spacetokensArray, SPLITTER );
    if ( nr_tokens <= 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: %s %s", INVALID_SURL_SPEC, CHK_CMD );
		return rei->status;
    }

    /*
     * Set up srm_getspacemd_input data structure
     */
    input.nbtokens      = nr_tokens;
    input.spacetokens   = spacetokensArray;

    status = srm_getspacemd_func( endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceMD: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
