#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"


char* resultString = NULL;

int srm_mkdir_func( char *endpoint, struct srm_mkdir_input *input ) {

    int status;

	struct srm_context context;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_mkdir() return 0 for success or -1 for error
     */
	status = srm_mkdir( &context, input );
    if ( (status != 0) && (context.errbuf != NULL)  ) {
        /*
         * Failure - we will use SRM context error buffer for message
         */
        resultString = (char* ) calloc( strlen(context.errbuf) + 1, sizeof(char) );
        if ( resultString == NULL) {
            return status;
        }
        strncpy( resultString, context.errbuf, strlen(context.errbuf) );
    }
    return status;
}

/**
 * \fn                  msiSrmMkdir
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URL of new directory
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmMkDir( msParam_t *endpoint, msParam_t *surl, ruleExecInfo_t *rei ) {

	struct srm_mkdir_input mkdir_input;

    if ( rei == NULL
         || rei->rsComm == NULL ) {

        rodsLog ( LOG_ERROR, "srmMkdir: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointers
     */
    if ( endpoint == NULL
         || surl == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmMkdir: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || surl->type == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmMkdir: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surl->type, STR_MS_T ) != 0 ) {

		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmMkdir: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_mkdir_input data structure
     */
    mkdir_input.dir_name    = (char * ) surl->inOutStruct;     // SURL of new directory

    if ( srm_mkdir_func( (char *) endpoint->inOutStruct, &mkdir_input ) == 0 ) {
        /* SUCCESS */
        rei->status = 1;
    }
    else {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        if ( resultString != NULL ) {
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmMkdir: Could not create any directory. %s\n%s", CHK_CMD, resultString );
        }
        else {
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmMkdir: Could not create any directory. %s", CHK_CMD );
        }
    }
    return rei->status;
}
