#include <errno.h>

#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"


char *resultString;

int srm_ping_func( char *url ) {

	int result;
    int len;

	struct srm_context context;
	struct srm_ping_output output;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, url );

	result = srm_ping( &context, &output );

	if ( result == 0 ) {
		if ( output.versioninfo != NULL ) {
            len = strlen(OK) + strlen("SRM Version: ") + strlen(output.versioninfo) + strlen(".\n");
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%sSRM Version: %s.\n", OK, output.versioninfo );
        }
        else {
            len = strlen(OK);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s", OK );
        }
	}
    else {
        /*
         * Something went wrong while setting up connection to web service
         */
        if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
        }
	}
	return result;
}

/**
 * \fn                  msiSrmPing
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmPing( msParam_t *endpoint, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmPing: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    if ( endpoint == NULL
         || resultParameter == NULL ) {
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPing: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL ) {
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPing: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPing: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    status = srm_ping_func( (char *) endpoint->inOutStruct );
    if ( status == 0 ) {
        /* SUCCESS */
	    rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    else if ( status == OUT_OF_MEMORY_EXCEPTION ) {        
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPing: %s", OUT_OF_MEM );
    }
    else {
        /* OTHER FAILURE  */
	    rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "%s", resultString );
    }
    return rei->status;
}
